import * as _nuxt_schema from '@nuxt/schema';
import { Configuration } from 'webpack';
import { BrowserTracing } from '@sentry/tracing';
import { Options as Options$1, IntegrationClass } from '@sentry/types';
import * as PluggableIntegrations from '@sentry/integrations';
import { Integrations } from '@sentry/vue';
import { Options, TracingOptions } from '@sentry/vue/types/types';
import { SentryCliPluginOptions } from '@sentry/webpack-plugin';
import { NodeOptions, Handlers, Integrations as Integrations$1 } from '@sentry/node';
import * as SentryTypes from '@sentry/core';

type IntegrationsConfig<T extends Record<keyof T, IntegrationClass<unknown>>> = Partial<{
    [K in keyof T]: ConstructorParameters<T[K]>[0] | Record<string, never>
}>

type ClientIntegrations = IntegrationsConfig<typeof Integrations & typeof PluggableIntegrations>
type ServerIntegrations = IntegrationsConfig<typeof Integrations$1 & typeof PluggableIntegrations>

interface LazyConfiguration {
    chunkName?: string
    injectLoadHook?: boolean
    injectMock?: boolean
    mockApiMethods?: boolean | string[]
    webpackPrefetch?: boolean
    webpackPreload?: boolean
}

interface TracingConfiguration extends Pick<Options$1, 'tracesSampleRate'> {
    browserTracing?: Partial<BrowserTracing['options']>
    vueOptions?: Partial<TracingOptions>
}

interface ModuleConfiguration {
  clientConfig: Partial<Options> | string
  clientIntegrations: ClientIntegrations
  config: Options$1
  customClientIntegrations: string
  customServerIntegrations: string
  disableClientRelease: boolean
  disableClientSide: boolean
  disabled: boolean
  disableServerRelease: boolean
  disableServerSide: boolean
  dsn: string
  tracing: boolean | TracingConfiguration
  initialize: boolean
  lazy: boolean | LazyConfiguration
  logMockCalls: boolean
  /** See available options at https://github.com/getsentry/sentry-webpack-plugin */
  publishRelease: boolean | SentryCliPluginOptions
  runtimeConfigKey: string
  serverConfig: NodeOptions | string
  serverIntegrations: ServerIntegrations
  sourceMapStyle: Configuration['devtool']
  requestHandlerConfig: Handlers.RequestHandlerOptions
}

type DeepPartial<T> = {
  [P in keyof T]?: T[P] extends Array<infer I>
    ? Array<DeepPartial<I>>
    : DeepPartial<T[P]>;
}

type DeepPartialModuleConfiguration = DeepPartial<ModuleConfiguration>

// add type to Vue context
declare module 'vue/types/vue' {
  interface Vue {
    readonly $sentry: typeof SentryTypes
    $sentryLoad(): Promise<void>
    $sentryReady(): Promise<typeof SentryTypes>
  }
}

// App Context and NuxtAppOptions
declare module '@nuxt/types' {
  interface Context {
    readonly $sentry: typeof SentryTypes
    $sentryLoad(): Promise<void>
    $sentryReady(): Promise<typeof SentryTypes>
  }

  interface NuxtAppOptions {
    readonly $sentry: typeof SentryTypes
    $sentryLoad(): Promise<void>
    $sentryReady(): Promise<typeof SentryTypes>
  }
}

declare module '@nuxt/types/config/runtime' {
  interface NuxtRuntimeConfig {
    sentry?: DeepPartialModuleConfiguration
  }
}

// add types for Vuex Store
declare module 'vuex/types' {
  interface Store<S> {
    readonly $sentry: typeof SentryTypes
    $sentryLoad(): Promise<void>
    $sentryReady(): Promise<typeof SentryTypes>
  }
}

type ModuleOptions = DeepPartialModuleConfiguration;
type ModulePublicRuntimeConfig = DeepPartialModuleConfiguration;
type ModulePrivateRuntimeConfig = DeepPartialModuleConfiguration;
declare const _default: _nuxt_schema.NuxtModule<ModuleConfiguration>;

export { ModuleOptions, ModulePrivateRuntimeConfig, ModulePublicRuntimeConfig, _default as default };
